import 'dart:async';

import 'package:flutter/material.dart';
import 'package:pit_components/pit_components.dart';

abstract class AdvState<T extends StatefulWidget> extends State<T> {
  bool _firstRun = true;
  bool _processing = false;
  Color loadingBackgroundColor = Colors.black.withOpacity(0.3);
  bool withLoading = true;

  @override
  Widget build(BuildContext context) {
    if (_firstRun) {
      initStateWithContext(context);
      _firstRun = false;
    }

    return WillPopScope(
      child: advBuild(context),
      onWillPop: () async {
        return !isProcessing();
      },
    );
  }

  void initStateWithContext(BuildContext context) {}

  Widget advBuild(BuildContext context);

  bool isProcessing() => _processing;

  Future<void> process(Function f) async {
    setState(() {
      _processing = true;
    });

    OverlayEntry x = withLoading ? _showLoading() : null;
    await f();
    if (this.mounted) {
      setState(() {
        _processing = false;
      });
    }
    x?.remove();
  }

  OverlayEntry _showLoading() {
    OverlayEntry toastOverlay = _createLoadingOverlay();

    OverlayState overlay = Overlay.of(context);

    if (overlay == null) return null;
    overlay.insert(toastOverlay);

    return toastOverlay;
  }

  OverlayEntry _createLoadingOverlay() {
    return OverlayEntry(
      builder: (context) => Positioned(
            child: Container(
              color: loadingBackgroundColor,
              child: Center(
                child: Image.asset(
                  PitComponents.loadingAssetName,
                  height: PitComponents.loadingHeight,
                  width: PitComponents.loadingWidth,
                ),
              ),
            ),
          ),
    );
  }
}
